@@ -15,8 +15,12 @@
  */
 package org.uberfire.ext.wires.core.grids.client.widget.grid.impl;
 
+import java.util.ArrayDeque;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
+import java.util.Queue;
+import java.util.stream.Stream;
 
 import com.ait.lienzo.client.core.Context2D;
 import com.ait.lienzo.client.core.event.NodeMouseClickEvent;
@@ -28,9 +32,11 @@ import com.ait.lienzo.client.core.shape.Layer;
 import com.ait.lienzo.client.core.shape.Node;
 import com.ait.lienzo.client.core.types.BoundingBox;
 import com.ait.lienzo.client.core.types.Point2D;
+import org.uberfire.commons.data.Pair;
 import org.uberfire.ext.wires.core.grids.client.model.GridColumn;
 import org.uberfire.ext.wires.core.grids.client.model.GridData;
 import org.uberfire.ext.wires.core.grids.client.widget.context.GridBodyRenderContext;
+import org.uberfire.ext.wires.core.grids.client.widget.context.GridBoundaryRenderContext;
 import org.uberfire.ext.wires.core.grids.client.widget.context.GridHeaderRenderContext;
 import org.uberfire.ext.wires.core.grids.client.widget.dnd.GridWidgetDnDHandlersState;
 import org.uberfire.ext.wires.core.grids.client.widget.dom.HasDOMElementResources;
@@ -55,22 +61,28 @@ public class BaseGridWidget extends Group implements GridWidget {
 
     protected final SelectionsTransformer bodyTransformer;
     protected final SelectionsTransformer floatingColumnsTransformer;
-
     protected final BaseGridRendererHelper rendererHelper;
+    protected final Queue<Pair<Group, List<GridRenderer.RendererCommand>>> renderQueue = new ArrayDeque<>();
 
     //These are final as a reference is held by the ISelectionsTransformers
-    protected final List<GridColumn<?>> allColumns = new ArrayList<GridColumn<?>>();
-    protected final List<GridColumn<?>> bodyColumns = new ArrayList<GridColumn<?>>();
-    protected final List<GridColumn<?>> floatingColumns = new ArrayList<GridColumn<?>>();
-    private final CellSelectionManager cellSelectionManager;
+    protected final List<GridColumn<?>> allColumns = new ArrayList<>();
+    protected final List<GridColumn<?>> bodyColumns = new ArrayList<>();
+    protected final List<GridColumn<?>> floatingColumns = new ArrayList<>();
+
     protected GridData model;
     protected GridRenderer renderer;
     protected Group header = null;
     protected Group floatingHeader = null;
     protected Group body = null;
+    protected Group bodySelections = null;
     protected Group floatingBody = null;
+    protected Group floatingBodySelections = null;
+    protected Group boundary = null;
+    protected BaseGridRendererHelper.RenderingInformation renderingInformation;
+
     private Group selection = null;
     private boolean isSelected = false;
+    private final CellSelectionManager cellSelectionManager;
 
     public BaseGridWidget(final GridData model,
                           final GridSelectionManager selectionManager,
@@ -198,11 +210,10 @@ public class BaseGridWidget extends Group implements GridWidget {
     @Override
     public void select() {
         isSelected = true;
-        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
         if (renderingInformation == null) {
             return;
         }
-        assertSelectionWidget(renderingInformation);
+        assertSelectionWidget();
         add(selection);
     }
 
@@ -219,10 +230,12 @@ public class BaseGridWidget extends Group implements GridWidget {
         return isSelected;
     }
 
-    private void assertSelectionWidget(final BaseGridRendererHelper.RenderingInformation renderingInformation) {
-        this.selection = renderer.renderSelector(getWidth(),
-                                                 getHeight(),
-                                                 renderingInformation);
+    private void assertSelectionWidget() {
+        this.selection = new Group();
+        addCommandToRenderQueue(selection,
+                                renderer.renderSelector(getWidth(),
+                                                        getHeight(),
+                                                        renderingInformation));
     }
 
     /**
@@ -239,11 +252,8 @@ public class BaseGridWidget extends Group implements GridWidget {
     protected void drawWithoutTransforms(Context2D context,
                                          double alpha,
                                          BoundingBox bb) {
-        body = null;
-        header = null;
-        floatingBody = null;
-        floatingHeader = null;
-        if ((context.isSelection()) && (false == isListening())) {
+        final boolean isSelectionLayer = context.isSelection();
+        if (isSelectionLayer && (false == isListening())) {
             return;
         }
         alpha = alpha * getAttributes().getAlpha();
@@ -251,101 +261,34 @@ public class BaseGridWidget extends Group implements GridWidget {
         if (alpha <= 0) {
             return;
         }
-
         if (model.getColumns().isEmpty()) {
             return;
         }
 
         //Clear existing content
         this.removeAll();
-        this.allColumns.clear();
-        this.bodyColumns.clear();
-        this.floatingColumns.clear();
 
-        //If there's no RenderingInformation the GridWidget is not visible
-        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
-        if (renderingInformation == null) {
-            destroyDOMElementResources();
-            return;
-        }
-
-        final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
-        final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
-        final List<GridColumn<?>> allColumns = renderingInformation.getAllColumns();
-        final List<GridColumn<?>> bodyColumns = bodyBlockInformation.getColumns();
-        final List<GridColumn<?>> floatingColumns = floatingBlockInformation.getColumns();
-        final boolean isSelectionLayer = context.isSelection();
-
-        this.allColumns.addAll(allColumns);
-        this.bodyColumns.addAll(bodyColumns);
-        this.floatingColumns.addAll(floatingColumns);
-
-        //Signal columns to attach or detach rendering support
         if (!isSelectionLayer) {
-            for (GridColumn<?> column : model.getColumns()) {
-                if (bodyColumns.contains(column) || floatingColumns.contains(column)) {
-                    if (column instanceof HasMultipleDOMElementResources) {
-                        ((HasMultipleDOMElementResources) column).initialiseResources();
-                    }
-                } else if (column instanceof HasDOMElementResources) {
-                    ((HasDOMElementResources) column).destroyResources();
-                }
+            //If there's no RenderingInformation the GridWidget is not visible
+            this.renderingInformation = prepare();
+            if (renderingInformation == null) {
+                destroyDOMElementResources();
+                return;
             }
+            makeRenderingCommands();
         }
 
-        //Draw if required
-        if (this.bodyColumns.size() > 0) {
-            drawHeader(renderingInformation,
-                       isSelectionLayer);
+        layerRenderGroups();
 
-            if (model.getRowCount() > 0) {
-                drawBody(renderingInformation,
-                         isSelectionLayer);
-            }
-
-            if (body != null) {
-                add(body);
-            }
-            if (header != null) {
-                add(header);
-            }
-            if (body != null || header != null) {
-                add(renderGridBoundary(bodyColumns,
-                                       bodyBlockInformation.getX(),
-                                       renderingInformation));
-            }
-
-            if (floatingBody != null) {
-                add(floatingBody);
-            }
-            if (floatingHeader != null) {
-                add(floatingHeader);
-            }
-            if (floatingBody != null || floatingHeader != null) {
-                add(renderGridBoundary(floatingColumns,
-                                       floatingBlockInformation.getX(),
-                                       renderingInformation));
-            }
-
-            //Include selection indicator if required
-            if (isSelected) {
-                assertSelectionWidget(renderingInformation);
-                add(selection);
-            }
-        }
+        executeRenderQueueCommands(isSelectionLayer);
 
         //Signal columns to free any unused resources
         if (!isSelectionLayer) {
-            for (GridColumn<?> column : bodyColumns) {
-                if (column instanceof HasMultipleDOMElementResources) {
-                    ((HasMultipleDOMElementResources) column).freeUnusedResources();
-                }
-            }
-            for (GridColumn<?> column : floatingColumns) {
-                if (column instanceof HasMultipleDOMElementResources) {
-                    ((HasMultipleDOMElementResources) column).freeUnusedResources();
-                }
-            }
+            Stream.concat(bodyColumns.stream(),
+                          floatingColumns.stream())
+                    .filter(column -> column instanceof HasMultipleDOMElementResources)
+                    .map(column -> (HasMultipleDOMElementResources) column)
+                    .forEach(HasMultipleDOMElementResources::freeUnusedResources);
         }
 
         //Then render to the canvas
@@ -354,7 +297,39 @@ public class BaseGridWidget extends Group implements GridWidget {
                                     bb);
     }
 
-    void destroyDOMElementResources() {
+    private BaseGridRendererHelper.RenderingInformation prepare() {
+        this.body = null;
+        this.header = null;
+        this.floatingBody = null;
+        this.floatingHeader = null;
+        this.bodySelections = null;
+        this.floatingBodySelections = null;
+        this.boundary = null;
+        this.allColumns.clear();
+        this.bodyColumns.clear();
+        this.floatingColumns.clear();
+        this.renderQueue.clear();
+
+        //If there's no RenderingInformation the GridWidget is not visible
+        final BaseGridRendererHelper.RenderingInformation renderingInformation = rendererHelper.getRenderingInformation();
+        if (renderingInformation == null) {
+            return null;
+        }
+
+        final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
+        final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
+        final List<GridColumn<?>> allColumns = renderingInformation.getAllColumns();
+        final List<GridColumn<?>> bodyColumns = bodyBlockInformation.getColumns();
+        final List<GridColumn<?>> floatingColumns = floatingBlockInformation.getColumns();
+
+        this.allColumns.addAll(allColumns);
+        this.bodyColumns.addAll(bodyColumns);
+        this.floatingColumns.addAll(floatingColumns);
+
+        return renderingInformation;
+    }
+
+    private void destroyDOMElementResources() {
         for (GridColumn<?> column : model.getColumns()) {
             if (column.getColumnRenderer() instanceof HasDOMElementResources) {
                 ((HasDOMElementResources) column.getColumnRenderer()).destroyResources();
@@ -362,6 +337,94 @@ public class BaseGridWidget extends Group implements GridWidget {
         }
     }
 
+    private void makeRenderingCommands() {
+        //Signal columns to attach or detach rendering support
+        for (GridColumn<?> column : model.getColumns()) {
+            if (bodyColumns.contains(column) || floatingColumns.contains(column)) {
+                if (column instanceof HasMultipleDOMElementResources) {
+                    ((HasMultipleDOMElementResources) column).initialiseResources();
+                }
+            } else if (column instanceof HasDOMElementResources) {
+                ((HasDOMElementResources) column).destroyResources();
+            }
+        }
+
+        //Draw if required
+        if (bodyColumns.size() > 0) {
+
+            boundary = new Group();
+
+            drawHeader(renderingInformation);
+
+            if (model.getRowCount() > 0) {
+                drawBody(renderingInformation);
+            }
+        }
+
+        final int minVisibleRowIndex = renderingInformation.getMinVisibleRowIndex();
+        final int maxVisibleRowIndex = renderingInformation.getMaxVisibleRowIndex();
+        final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
+        final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
+
+        if (bodySelections != null) {
+            addCommandToRenderQueue(bodySelections,
+                                    renderSelectedRanges(bodyColumns,
+                                                         bodyBlockInformation.getX(),
+                                                         minVisibleRowIndex,
+                                                         maxVisibleRowIndex,
+                                                         bodyTransformer,
+                                                         renderingInformation));
+        }
+        if (floatingBodySelections != null) {
+            addCommandToRenderQueue(floatingBodySelections,
+                                    renderSelectedRanges(floatingColumns,
+                                                         floatingBlockInformation.getX(),
+                                                         minVisibleRowIndex,
+                                                         maxVisibleRowIndex,
+                                                         floatingColumnsTransformer,
+                                                         renderingInformation));
+        }
+        if (boundary != null) {
+            addCommandToRenderQueue(boundary,
+                                    renderGridBoundary(renderingInformation));
+        }
+        if (isSelected) {
+            assertSelectionWidget();
+        }
+    }
+
+    private void layerRenderGroups() {
+        //The order these are added ensures the parts overlap correctly
+        if (body != null) {
+            add(body);
+        }
+        if (bodySelections != null) {
+            add(bodySelections);
+        }
+        if (header != null) {
+            add(header);
+        }
+
+        if (floatingBody != null) {
+            add(floatingBody);
+        }
+        if (floatingBodySelections != null) {
+            add(floatingBodySelections);
+        }
+        if (floatingHeader != null) {
+            add(floatingHeader);
+        }
+
+        if (boundary != null) {
+            add(boundary);
+        }
+
+        //Include selection indicator if required
+        if (isSelected) {
+            add(selection);
+        }
+    }
+
     @Override
     public Group setVisible(final boolean visible) {
         if (!visible) {
@@ -374,13 +437,10 @@ public class BaseGridWidget extends Group implements GridWidget {
         return super.setVisible(visible);
     }
 
-    protected void drawHeader(final BaseGridRendererHelper.RenderingInformation renderingInformation,
-                              final boolean isSelectionLayer) {
+    protected void drawHeader(final BaseGridRendererHelper.RenderingInformation renderingInformation) {
         final List<GridColumn<?>> allColumns = renderingInformation.getAllColumns();
         final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
         final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
-        final List<GridColumn<?>> bodyColumns = bodyBlockInformation.getColumns();
-        final List<GridColumn<?>> floatingColumns = floatingBlockInformation.getColumns();
 
         final double headerX = bodyBlockInformation.getX();
         final double headerY = bodyBlockInformation.getHeaderY();
@@ -390,34 +450,37 @@ public class BaseGridWidget extends Group implements GridWidget {
         //Add Header, if applicable
         final boolean addFixedHeader = renderingInformation.isFixedHeader();
         final boolean addFloatingHeader = renderingInformation.isFloatingHeader();
+
         if (addFixedHeader || addFloatingHeader) {
-            header = renderGridHeaderWidget(allColumns,
-                                            bodyColumns,
-                                            isSelectionLayer,
-                                            renderingInformation);
-            header.setX(headerX);
-            if (addFloatingHeader) {
-                header.setY(headerY);
+            //Render header for body columns, if required
+            if (bodyColumns.size() > 0) {
+                header = new Group();
+                header.setX(headerX);
+                addCommandsToRenderQueue(header,
+                                         renderGridHeaderWidget(allColumns,
+                                                                bodyColumns,
+                                                                renderingInformation));
+
+                if (addFloatingHeader) {
+                    header.setY(headerY);
+                }
             }
 
-            //Draw floating header columns if required
+            //Render header for floating columns, if required
             if (floatingColumns.size() > 0) {
-                floatingHeader = renderGridHeaderWidget(floatingColumns,
-                                                        floatingColumns,
-                                                        isSelectionLayer,
-                                                        renderingInformation);
-                floatingHeader.setX(floatingHeaderX);
-                floatingHeader.setY(floatingHeaderY);
+                floatingHeader = new Group();
+                floatingHeader.setX(floatingHeaderX).setY(floatingHeaderY);
+                addCommandsToRenderQueue(floatingHeader,
+                                         renderGridHeaderWidget(floatingColumns,
+                                                                floatingColumns,
+                                                                renderingInformation));
             }
         }
     }
 
-    protected void drawBody(final BaseGridRendererHelper.RenderingInformation renderingInformation,
-                            final boolean isSelectionLayer) {
+    protected void drawBody(final BaseGridRendererHelper.RenderingInformation renderingInformation) {
         final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
         final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
-        final List<GridColumn<?>> bodyColumns = bodyBlockInformation.getColumns();
-        final List<GridColumn<?>> floatingColumns = floatingBlockInformation.getColumns();
 
         final double bodyX = bodyBlockInformation.getX();
         final double bodyY = bodyBlockInformation.getBodyY();
@@ -427,68 +490,79 @@ public class BaseGridWidget extends Group implements GridWidget {
         final int minVisibleRowIndex = renderingInformation.getMinVisibleRowIndex();
         final int maxVisibleRowIndex = renderingInformation.getMaxVisibleRowIndex();
 
-        body = renderGridBodyWidget(bodyColumns,
-                                    bodyBlockInformation.getX(),
-                                    minVisibleRowIndex,
-                                    maxVisibleRowIndex,
-                                    isSelectionLayer,
-                                    bodyTransformer,
-                                    renderingInformation);
-        body.setX(bodyX);
-        body.setY(bodyY);
+        //Render body columns, if required
+        if (bodyColumns.size() > 0) {
+            body = new Group();
+            body.setX(bodyX).setY(bodyY);
+            bodySelections = new Group();
+            bodySelections.setX(bodyX).setY(bodyY);
 
-        //Include selected ranges of cells
-        if (!isSelectionLayer) {
-            body.add(renderSelectedRanges(bodyColumns,
-                                          bodyBlockInformation.getX(),
-                                          minVisibleRowIndex,
-                                          maxVisibleRowIndex,
-                                          bodyTransformer,
-                                          renderingInformation));
+            addCommandsToRenderQueue(body,
+                                     renderGridBodyWidget(bodyColumns,
+                                                          bodyBlockInformation.getX(),
+                                                          minVisibleRowIndex,
+                                                          maxVisibleRowIndex,
+                                                          bodyTransformer,
+                                                          renderingInformation));
         }
 
-        //Render floating columns
+        //Render floating columns, if required
         if (floatingColumns.size() > 0) {
-            floatingBody = renderGridBodyWidget(floatingColumns,
-                                                floatingBlockInformation.getX(),
-                                                minVisibleRowIndex,
-                                                maxVisibleRowIndex,
-                                                isSelectionLayer,
-                                                floatingColumnsTransformer,
-                                                renderingInformation);
-            floatingBody.setX(floatingBodyX);
-            floatingBody.setY(floatingBodyY);
+            floatingBody = new Group();
+            floatingBody.setX(floatingBodyX).setY(floatingBodyY);
+            floatingBodySelections = new Group();
+            floatingBodySelections.setX(floatingBodyX).setY(floatingBodyY);
 
-            //Include selected ranges of cells
-            if (!isSelectionLayer) {
-                floatingBody.add(renderSelectedRanges(floatingColumns,
-                                                      floatingBlockInformation.getX(),
-                                                      minVisibleRowIndex,
-                                                      maxVisibleRowIndex,
-                                                      floatingColumnsTransformer,
-                                                      renderingInformation));
-            }
+            addCommandsToRenderQueue(floatingBody,
+                                     renderGridBodyWidget(floatingColumns,
+                                                          floatingBlockInformation.getX(),
+                                                          minVisibleRowIndex,
+                                                          maxVisibleRowIndex,
+                                                          floatingColumnsTransformer,
+                                                          renderingInformation));
         }
     }
 
+    protected void addCommandToRenderQueue(final Group parent,
+                                           final GridRenderer.RendererCommand command) {
+        addCommandsToRenderQueue(parent, Collections.singletonList(command));
+    }
+
+    protected void addCommandsToRenderQueue(final Group parent,
+                                            final List<GridRenderer.RendererCommand> commands) {
+        renderQueue.add(new Pair<>(parent, commands));
+    }
+
+    protected void executeRenderQueueCommands(final boolean isSelectionLayer) {
+        renderQueue.stream()
+                .forEach(p -> p.getK2()
+                        .forEach(c -> c.execute(new GridRenderer.GridRendererContext() {
+                            @Override
+                            public Group getGroup() {
+                                return p.getK1();
+                            }
+
+                            @Override
+                            public boolean isSelectionLayer() {
+                                return isSelectionLayer;
+                            }
+                        })));
+    }
+
     /**
      * Render the Widget's Header and append to this Group.
      * @param allColumns All columns in the model.
      * @param blockColumns The columns to render for a block.
-     * @param isSelectionLayer Is the SelectionLayer being rendered.
      */
-    protected Group renderGridHeaderWidget(final List<GridColumn<?>> allColumns,
-                                           final List<GridColumn<?>> blockColumns,
-                                           final boolean isSelectionLayer,
-                                           final BaseGridRendererHelper.RenderingInformation renderingInformation) {
+    protected List<GridRenderer.RendererCommand> renderGridHeaderWidget(final List<GridColumn<?>> allColumns,
+                                                                        final List<GridColumn<?>> blockColumns,
+                                                                        final BaseGridRendererHelper.RenderingInformation renderingInformation) {
         final GridHeaderRenderContext context = new GridHeaderRenderContext(allColumns,
-                                                                            blockColumns,
-                                                                            isSelectionLayer);
-        final Group g = renderer.renderHeader(model,
-                                              context,
-                                              rendererHelper,
-                                              renderingInformation);
-        return g;
+                                                                            blockColumns);
+        return renderer.renderHeader(model,
+                                     context,
+                                     rendererHelper,
+                                     renderingInformation);
     }
 
     /**
@@ -497,16 +571,14 @@ public class BaseGridWidget extends Group implements GridWidget {
      * @param absoluteColumnOffsetX Absolute offset from Grid's X co-ordinate to render first column in block.
      * @param minVisibleRowIndex The index of the first visible row.
      * @param maxVisibleRowIndex The index of the last visible row.
-     * @param isSelectionLayer Is the SelectionLayer being rendered.
      * @param transformer SelectionTransformer in operation.
      */
-    protected Group renderGridBodyWidget(final List<GridColumn<?>> blockColumns,
-                                         final double absoluteColumnOffsetX,
-                                         final int minVisibleRowIndex,
-                                         final int maxVisibleRowIndex,
-                                         final boolean isSelectionLayer,
-                                         final SelectionsTransformer transformer,
-                                         final BaseGridRendererHelper.RenderingInformation renderingInformation) {
+    protected List<GridRenderer.RendererCommand> renderGridBodyWidget(final List<GridColumn<?>> blockColumns,
+                                                                      final double absoluteColumnOffsetX,
+                                                                      final int minVisibleRowIndex,
+                                                                      final int maxVisibleRowIndex,
+                                                                      final SelectionsTransformer transformer,
+                                                                      final BaseGridRendererHelper.RenderingInformation renderingInformation) {
         final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
         final double floatingX = floatingBlockInformation.getX();
         final double floatingWidth = floatingBlockInformation.getWidth();
@@ -521,36 +593,50 @@ public class BaseGridWidget extends Group implements GridWidget {
                                                                         minVisibleRowIndex,
                                                                         maxVisibleRowIndex,
                                                                         blockColumns,
-                                                                        isSelectionLayer,
                                                                         getViewport().getTransform(),
                                                                         renderer,
                                                                         transformer);
-        final Group g = renderer.renderBody(model,
-                                            context,
-                                            rendererHelper,
-                                            renderingInformation);
-        return g;
+        return renderer.renderBody(model,
+                                   context,
+                                   rendererHelper,
+                                   renderingInformation);
     }
 
     /**
      * Render the selected ranges and append to the Body Group.
-     * @param blockColumns The columns to render.
-     * @param absoluteColumnOffsetX Absolute offset from Grid's X co-ordinate to render first column in block.
      * @param renderingInformation Calculated rendering information supporting rendering.
      * @return A Group containing the boundary.
      */
-    protected Group renderGridBoundary(final List<GridColumn<?>> blockColumns,
-                                       final double absoluteColumnOffsetX,
-                                       final BaseGridRendererHelper.RenderingInformation renderingInformation) {
+    protected GridRenderer.RendererCommand renderGridBoundary(final BaseGridRendererHelper.RenderingInformation renderingInformation) {
+        double x = 0;
+        final List<GridColumn<?>> allColumns = new ArrayList<>();
+        final BaseGridRendererHelper.RenderingBlockInformation bodyBlockInformation = renderingInformation.getBodyBlockInformation();
+        final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
+
+        if (body != null || header != null) {
+            allColumns.addAll(bodyColumns);
+            x = bodyBlockInformation.getX();
+        }
+        if (floatingBody != null || floatingHeader != null) {
+            allColumns.addAll(floatingColumns);
+            x = floatingBlockInformation.getX();
+        }
         final double headerYOffset = (header == null ? 0.0 : header.getY());
         final double headerRowsYOffset = renderingInformation.getHeaderRowsYOffset();
-        final Group boundary = renderer.renderGridBoundary(rendererHelper.getWidth(blockColumns),
-                                                           getHeight() - headerRowsYOffset - headerYOffset);
-        boundary.setY(headerRowsYOffset + headerYOffset);
-        boundary.setX(absoluteColumnOffsetX);
-        add(boundary);
+        final double y = headerRowsYOffset + headerYOffset;
 
-        return boundary;
+        final double height = getHeight() - headerRowsYOffset - headerYOffset;
+        double width = rendererHelper.getWidth(allColumns);
+        if (!floatingColumns.isEmpty()) {
+            width = width - (floatingBlockInformation.getX() + rendererHelper.getWidth(floatingColumns) - bodyBlockInformation.getX());
+        }
+
+        final GridBoundaryRenderContext context = new GridBoundaryRenderContext(x,
+                                                                                y,
+                                                                                width,
+                                                                                height);
+
+        return renderer.renderGridBoundary(context);
     }
 
     /**
@@ -562,12 +648,12 @@ public class BaseGridWidget extends Group implements GridWidget {
      * @param transformer SelectionTransformer in operation.
      * @return
      */
-    protected Group renderSelectedRanges(final List<GridColumn<?>> blockColumns,
-                                         final double absoluteColumnOffsetX,
-                                         final int minVisibleRowIndex,
-                                         final int maxVisibleRowIndex,
-                                         final SelectionsTransformer transformer,
-                                         final BaseGridRendererHelper.RenderingInformation renderingInformation) {
+    protected GridRenderer.RendererCommand renderSelectedRanges(final List<GridColumn<?>> blockColumns,
+                                                                final double absoluteColumnOffsetX,
+                                                                final int minVisibleRowIndex,
+                                                                final int maxVisibleRowIndex,
+                                                                final SelectionsTransformer transformer,
+                                                                final BaseGridRendererHelper.RenderingInformation renderingInformation) {
         final BaseGridRendererHelper.RenderingBlockInformation floatingBlockInformation = renderingInformation.getFloatingBlockInformation();
         final double floatingX = floatingBlockInformation.getX();
         final double floatingWidth = floatingBlockInformation.getWidth();
@@ -582,14 +668,12 @@ public class BaseGridWidget extends Group implements GridWidget {
                                                                         minVisibleRowIndex,
                                                                         maxVisibleRowIndex,
                                                                         blockColumns,
-                                                                        false,
                                                                         getViewport().getTransform(),
                                                                         renderer,
                                                                         transformer);
-        final Group g = renderer.renderSelectedCells(model,
-                                                     context,
-                                                     rendererHelper);
-        return g;
+        return renderer.renderSelectedCells(model,
+                                            context,
+                                            rendererHelper);
     }
 
     @Override
@@ -643,7 +727,7 @@ public class BaseGridWidget extends Group implements GridWidget {
     }
 
     @Override
-    public boolean startEditingCell(final Point2D ap) {
-        return cellSelectionManager.startEditingCell(ap);
+    public boolean startEditingCell(final Point2D rp) {
+        return cellSelectionManager.startEditingCell(rp);
     }
 }
